/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include <glib/gprintf.h>

#include <libxfce4util/libxfce4util.h>

#define CONFFILE "/cdrom/boot.cfg"

static void
gen_str (gpointer data, gpointer userdata)
{
    gchar *item = (gchar *) data;
    gchar **str = (gchar **) userdata;
    gchar *new_str = NULL;
    int size;
    if (*str)
    {
        size = strlen (item) + strlen (*str) + 2; /* space and \0 */
        new_str = (gchar *) malloc (sizeof (gchar) * size);
        g_sprintf (new_str, "%s %s", *str, item);
    }
    else
    {
        size = strlen (item) + 1; /* final \0 */
        new_str = g_strdup (item);
    }
    g_free (*str);
    *str = new_str;
}

static void
usage ()
{
    printf("Usage:\n"
           "services-blacklist help|list|add|del [services]\n"
           "   help: show this message\n"
           "   list: display the blacklisted services\n"
           "   add : add the listed service(s) to the blacklist\n"
           "   del : delete the listed service(s) from the blacklist\n");
}

int main (int argc, gchar **argv)
{
    if (getuid() != 0) {
        g_fprintf (stderr, _("You don't appear to be root. Exiting.\n"));
        return 1;
    }

    XfceRc *rc;
    gchar *str = NULL;
    const gchar *cmd = argv[1];

    rc = xfce_rc_simple_open (CONFFILE, TRUE);
    if (!rc)
    {
        g_fprintf (stderr, _("Unable to read the config file. Exiting.\n"));
        return 1;
    }

    str = g_shell_unquote (xfce_rc_read_entry (rc, "SERVICES_BLACKLIST", " "), NULL);
    xfce_rc_close (rc);

    if (!str)
    {
        g_fprintf (stderr, _("Unable to read the blacklist. Exiting.\n"));
        return 1;
    }

    if (argc == 2)
    {
        if (!strcmp (argv[1], "help"))
        {
            usage ();
            g_free (str);
            return 0;
        } else
        if (!strcmp (argv[1], "list")) {
            g_printf (_("blacklisted services: %s\n"), str);
            g_free (str);
            return 0;
        }
    }

    if (argc < 3)
    {
        usage ();
        return 1;
    }

    int i;
    GList *l = NULL;
    gchar **blacklist = NULL;
    blacklist = g_strsplit (str, " ", 0);
    int size = g_strv_length (blacklist);

    for (i=0;i<size;i++)
        l = g_list_append (l, blacklist[i]);
    g_free (str);
    str = NULL;


    for (i=2;i<argc;i++)
    {
        if (!strcmp ("add", cmd))
        {
            l = g_list_append (l, argv[i]);
        } else
        if (!strcmp ("del", cmd))
        {
            int j;
            for (j=0;j<size;j++)
                if (!strcmp (argv[i], blacklist[j]))
                    l = g_list_remove (l, blacklist[j]);
        }
    }

    g_list_foreach (l, gen_str, &str);
    gchar *data = g_shell_quote (str);
    rc = xfce_rc_simple_open (CONFFILE, FALSE);
    if (!rc)
    {
        g_fprintf (stderr, _("Unable to write the config file. Exiting.\n"));
        g_free (data);
        g_free (str);
        g_list_free (l);
        g_strfreev (blacklist);

        return 1;
    }
    xfce_rc_write_entry (rc, "SERVICES_BLACKLIST", data);
    xfce_rc_close (rc);

    /* free */
    g_free (data);
    g_free (str);
    g_list_free (l);
    g_strfreev (blacklist);

    return 0;
}
